#!/usr/bin/python
#
# Disable trackpoint device when keyboard is used.
#
# Derived from original by gcla
#  https://bugs.launchpad.net/ubuntu/+source/linux/+bug/606238/comments/41
#   it was derived: https://bugs.launchpad.net/ubuntu/+source/linux/+bug/606238/comments/39
#
# Igor Mega (I'm not a python programmer :( )
#
# No warranty - use at your own risk.
#
# Directly accesses input devices, so must be run as root
#   e.g. sudo kbddetector.py 

import os, sys, signal, subprocess, struct, ctypes, re

# Key codes that should not disable the trackpoint since they may be
# used in combination, see /usr/include/linux/input.h.
MODIFIER_KEYS = ( 29, 56, 219, 184, 157, 42, 54 )

_32bit = ctypes.sizeof(ctypes.c_voidp) == 4
_eventFormat = _32bit and "=LLHHl" or "=qqHHl"
_eventFormatSize = struct.calcsize(_eventFormat)


class Trackpoint:
    def __init__(self, deviceName):
        self._deviceName = deviceName
        self._state = 1
    
    def switch(self, newState):
        subprocess.call(["/usr/bin/xinput",
                         "set-int-prop",
                         self._deviceName,
                         "Device Enabled",
                         "8",
                         newState and "1" or "0"])
            
        self._state = newState
        print newState and "ON" or "OFF"

    def on(self): self._state == 0 and self.switch(1)
    def off(self): self._state == 1 and self.switch(0)

def _parseDevices():
    mouse = None
    keyboardEvent = None
    mousestr = "N: Name=\"("
    mousestr += sys.argv[1]
    mousestr += ")\""
    searchMouse = re.compile(mousestr)
    kbdstr = "N: Name=\""
    kbdstr += sys.argv[2]
    kbdstr += "\""
    searchKbd = re.compile(kbdstr)

    handlersRE = re.compile("H: Handlers=(.*)")            
    eventRE = re.compile("event(\d)")

    f = open("/proc/bus/input/devices")

    try:
        while True:
            line = f.readline()

            if not line:
                break
            if not mouse: 
                match = searchMouse.search(line)
                if match:
                     mouse = match.group(1) 
                     if keyboardEvent:
                           break
                     continue

            if not keyboardEvent:
                match = searchKbd.search(line)
                if match:
                     while True:
                        line = f.readline()
                        if not line:
                           break
                        if len(line) < 4:
                           break
                        match = handlersRE.match(line)                        
                        if match:
                             handlers = match.group(1)
                             match = eventRE.search(handlers)
                             if match: 
                                  keyboardEvent = int(match.group(1))
                                  break                     
                     if mouse:
                        if keyboardEvent: 
                                 break
                     continue
                match = handlersRE.match(line)

    finally:
        f.close()
    
    return mouse, keyboardEvent
    

if __name__ == "__main__":

    if len(sys.argv) != 5:
         print "Three arguments needed..."
         print "  1st mouse name"
         print "  2nd keyboard name"
         print "  3rd interval - 0.01 - 3.0, optimal value: 0.1-0.3"
         print "  4th state on Terminate 0 or 1"
         print " Use xinput list for these names (for 1st, 2nd)"
         print " Use 5 and more parameters for devices search test"
    user_time = 0.1
    try:
        user_time = float(sys.argv[3])
    except ValueError:
        print "Error: 3rd parameter parse to real number"
        sys.exit(3)
    if user_time < 0.01:
        print "Error: 3rd parameter is lower then 0.01"
        sys.exit(3)
    if user_time > 3.0:
        print "Error: 3rd parameter is grather then 3.0"
        sys.exit(3)
    mouse, keyboardEvent =  _parseDevices()
    if len(sys.argv) > 5:
         print "----------------"
         print "Test devices search"
         if mouse: 
               print "Mouse found" 
         else: print "Mouse NOT found"
         if keyboardEvent: 
               print "Keyboard found" 
         else: print "Keyboard NOT found"
         print "----------------"
         sys.exit(3)
    if len(sys.argv) != 5:
         sys.exit(3)

    if not mouse or not keyboardEvent: 
       print "Failed to find devices"
       sys.exit(3)
        
    trackpoint = Trackpoint(mouse)

    def handler(signum, frame):
        if signum != signal.SIGALRM:
            trackpoint.switch(sys.argv[4])
            print "Terminating."
            sys.exit(0)
        else:
            trackpoint.on()
    
    [signal.signal(s, handler) for s in (signal.SIGTERM,
                                         signal.SIGQUIT,
                                         signal.SIGINT,
                                         signal.SIGALRM)]

    fd = os.open("/dev/input/event%d" % keyboardEvent, os.O_RDONLY)
    
    while True:
        try:
            event = os.read(fd, _eventFormatSize)
            _, _, etype, code, value = struct.unpack(_eventFormat, event)

            if etype == 1 and code not in MODIFIER_KEYS: trackpoint.off()              
            signal.setitimer(signal.ITIMER_REAL, user_time, 0)
                  
        except OSError:
            pass
